/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Parser.java 20829 2011-02-03 10:25:07Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.parser;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.ow2.jonas.autostart.utility.Utility;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * This class gets the lastest stable version of JOnAS.
 * @author skabore
 */
public class Parser {

    /**
     * The XML document to parse.
     */
    private Document document = null;

    /**
     * Version requested.
     */
    private Versions version;

    /**
     * The itemName.
     */
    private String itemName = null;

    /**
     * Build a new Parser with the following metadataUrl, properties, logger and
     * item.
     * @param metadataUrl the metadataurl
     * @param selectedVersion the selected version
     * @param item the item name
     */
    public Parser(final String metadataUrl, final String selectedVersion, final String item) {
        /**
         * The maven metadata XML document.
         */
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        this.itemName = item;

        String userVersion = selectedVersion;
        if ("snapshot".equals(userVersion)) {
            this.version = Versions.SNAPSHOT;
        } else if ("stable".equals(userVersion)) {
            this.version = Versions.STABLE;
        } else if ("milestone".equals(userVersion)) {
            this.version = Versions.MILESTONE;
        } else { // Default version
            this.version = Versions.STABLE;
        }

        try {
            DocumentBuilder documentBuilder = null;
            try {
                documentBuilder = documentBuilderFactory.newDocumentBuilder();
                this.document = documentBuilder.parse(metadataUrl);
            } catch (ParserConfigurationException ex) {
                Logger.getLogger(Parser.class.getName()).log(Level.SEVERE, null, ex);
            } catch (SAXException ex) {
                Logger.getLogger(Parser.class.getName()).log(Level.SEVERE, null, ex);
            }
        } catch (MalformedURLException ex) {
            Logger.getLogger(Utility.class.getName()).log(Level.OFF, ex.getMessage());
        } catch (IOException ex) {
            Logger.getLogger(Utility.class.getName()).log(Level.OFF, ex.getMessage());
        }
    }

    /**
     * This method parse the XML document and return the latest version child.
     * @return the last version.
     */
    public String getLatestVersion() {
        if (this.document != null) {
            String ret = "";
            List<String> versions = new ArrayList<String>();
            List<String> stableVersions = new ArrayList<String>();
            NodeList name = this.document.getElementsByTagName("version");
            for (int i = 0; i < name.getLength(); i++) {
                Node node = name.item(i);
                String tmpVersion = node.getTextContent();
                // Drop item versions or branch
                if (tmpVersion.contains(this.itemName.toUpperCase()) || tmpVersion.contains("branch")) {
                    continue;
                }
                // stable version
                if (!tmpVersion.contains("-")) {
                    stableVersions.add(tmpVersion);
                }
                versions.add(tmpVersion);
            }
            // Sort
            Collections.sort(versions);
            Collections.sort(stableVersions);

            // Snapshot/milestone case : latest version (of snapshot or release
            // repository)
            if ((this.version == Versions.SNAPSHOT) || (this.version == Versions.MILESTONE)) {
                ret = versions.get(versions.size() - 1);
            } else if ((this.version == Versions.STABLE)) {
                ret = stableVersions.get(stableVersions.size() - 1);
            }
            return ret;
        }
        return "";
    }

    /**
     * This method parse the XML document and return all latest versions child.
     * @return the list of versions.
     */
    public ArrayList<String> getAllLatestVersions() {
        ArrayList<String> versions = new ArrayList<String>();
        if (this.document != null) {
            NodeList name = this.document.getElementsByTagName("version");
            for (int i = 0; i < name.getLength(); i++) {
                Node node = name.item(i);
                String tmpVersion = node.getTextContent();
                // Drop itemName versions or branch
                if (tmpVersion.contains(this.itemName.toUpperCase()) || tmpVersion.contains("branch")) {
                    continue;
                }
                versions.add(tmpVersion);
            }
            // Sort
            Collections.sort(versions);
        }
        return versions;
    }
}

/**
 * Versions of JOnAS.
 * @author Florent Benoit
 */
enum Versions {

    /**
     * Stable.
     */
    STABLE,
    /**
     * Milestone.
     */
    MILESTONE,
    /**
     * Snapshot.
     */
    SNAPSHOT;
}
